@@ -112,7 +112,6 @@ sub form {
     };
 } # END form
 
-
 1;
 
 =head1 NAME
@@ -123,16 +122,13 @@ Contact::GEN::Number - generated support module for Contact::Number
 
 In Contact::Number:
 
-    use Contact::GEN::Number qw(
-        do_main
-        form
-    );
+    use base 'Contact::GEN::Number';
 
 =head1 DESCRIPTION
 
 This module was generated by bigtop and IS subject to regeneration.
 Use it in Contact::Number to provide the methods below.
-They are exported by default.
+Feel free to override them.
 
 =head1 METHODS
 
